Class: LLM::Anthropic
- Extended by:
- Utils
- Includes:
- RequestAdapter
- Defined in:
- lib/llm/providers/anthropic.rb,
lib/llm/providers/anthropic/files.rb,
lib/llm/providers/anthropic/utils.rb,
lib/llm/providers/anthropic/models.rb,
lib/llm/providers/anthropic/error_handler.rb,
lib/llm/providers/anthropic/stream_parser.rb,
lib/llm/providers/anthropic/request_adapter.rb,
lib/llm/providers/anthropic/response_adapter.rb
Overview
The Anthropic class implements a provider for [Anthropic](www.anthropic.com).
Defined Under Namespace
Modules: RequestAdapter, ResponseAdapter, Utils Classes: ErrorHandler, Files, Models, StreamParser
Constant Summary collapse
- HOST =
"api.anthropic.com"
Instance Method Summary collapse
-
#assistant_role ⇒ String
Returns the role of the assistant in the conversation.
-
#complete(prompt, params = {}) ⇒ LLM::Response
Provides an interface to the chat completions API.
-
#default_model ⇒ String
Returns the default model for chat completions.
-
#files ⇒ LLM::Anthropic::Files
Provides an interface to Anthropic’s files API.
-
#initialize ⇒ Anthropic
constructor
A new instance of Anthropic.
-
#models ⇒ LLM::Anthropic::Models
Provides an interface to Anthropic’s models API.
-
#name ⇒ Symbol
Returns the provider’s name.
- #server_tools ⇒ String => LLM::ServerTool
-
#tool_role ⇒ Symbol
Anthropic expects tool results to be sent as user messages containing ‘tool_result` content blocks rather than a distinct `tool` role.
-
#web_search(query:) ⇒ LLM::Response
A convenience method for performing a web search using the Anthropic web search tool.
Methods included from Utils
Methods included from RequestAdapter
Methods inherited from Provider
#audio, #chat, #developer_role, #embed, #images, #inspect, #interrupt!, #moderations, #persist!, #respond, #responses, #schema, #server_tool, #streamable?, #system_role, #tracer, #tracer=, #user_role, #vector_stores, #with
Constructor Details
Instance Method Details
#assistant_role ⇒ String
Returns the role of the assistant in the conversation. Usually “assistant” or “model”
80 81 82 |
# File 'lib/llm/providers/anthropic.rb', line 80 def assistant_role "assistant" end |
#complete(prompt, params = {}) ⇒ LLM::Response
Provides an interface to the chat completions API
52 53 54 55 56 57 58 59 60 |
# File 'lib/llm/providers/anthropic.rb', line 52 def complete(prompt, params = {}) params, stream, tools, role = normalize_complete_params(params) req = build_complete_request(prompt, params, role) res, span, tracer = execute(request: req, stream: stream, operation: "chat", model: params[:model]) res = ResponseAdapter.adapt(res, type: :completion) .extend(Module.new { define_method(:__tools__) { tools } }) tracer.on_request_finish(operation: "chat", model: params[:model], res:, span:) res end |
#default_model ⇒ String
Returns the default model for chat completions
97 98 99 |
# File 'lib/llm/providers/anthropic.rb', line 97 def default_model "claude-sonnet-4-20250514" end |
#files ⇒ LLM::Anthropic::Files
Provides an interface to Anthropic’s files API
74 75 76 |
# File 'lib/llm/providers/anthropic.rb', line 74 def files LLM::Anthropic::Files.new(self) end |
#models ⇒ LLM::Anthropic::Models
Provides an interface to Anthropic’s models API
66 67 68 |
# File 'lib/llm/providers/anthropic.rb', line 66 def models LLM::Anthropic::Models.new(self) end |
#name ⇒ Symbol
Returns the provider’s name
38 39 40 |
# File 'lib/llm/providers/anthropic.rb', line 38 def name :anthropic end |
#server_tools ⇒ String => LLM::ServerTool
This method includes certain tools that require configuration through a set of options that are easier to set through the LLM::Provider#server_tool method.
108 109 110 111 112 113 114 |
# File 'lib/llm/providers/anthropic.rb', line 108 def server_tools { bash: server_tool(:bash, type: "bash_20250124"), web_search: server_tool(:web_search, type: "web_search_20250305", max_uses: 5), text_editor: server_tool(:str_replace_based_edit_tool, type: "text_editor_20250728", max_characters: 10_000) } end |
#tool_role ⇒ Symbol
Anthropic expects tool results to be sent as user messages containing ‘tool_result` content blocks rather than a distinct `tool` role.
89 90 91 |
# File 'lib/llm/providers/anthropic.rb', line 89 def tool_role :user end |
#web_search(query:) ⇒ LLM::Response
A convenience method for performing a web search using the Anthropic web search tool.
125 126 127 |
# File 'lib/llm/providers/anthropic.rb', line 125 def web_search(query:) ResponseAdapter.adapt(complete(query, tools: [server_tools[:web_search]]), type: :web_search) end |