Class: LLM::Compactor::Null

Inherits:
LLM::Compactor show all
Defined in:
lib/llm/compactor/null.rb

Overview

An LLM::Compactor::Null is a compactor that does nothing. It is used as the default when no compactor strategy is configured.

All methods return nil and produce no side effects.

Instance Attribute Summary

Attributes inherited from LLM::Compactor

#ctx

Instance Method Summary collapse

Methods inherited from LLM::Compactor

#initialize

Constructor Details

This class inherits a constructor from LLM::Compactor

Instance Method Details

#call(**opts) ⇒ nil

Parameters:

  • opts (Hash)

    Ignored

Returns:

  • (nil)


15
16
17
# File 'lib/llm/compactor/null.rb', line 15

def call(**opts)
  nil
end