Module: LLM::Sequel::Agent::InstanceMethods

Defined in:
lib/llm/sequel/agent.rb

Instance Method Summary collapse

Instance Method Details

#repl(**params) ⇒ void

Note:

This method does not persist to the database, but it can inspect and alter runtime state in a way that is temporary.

This method returns an undefined value.

Parameters:

  • tracer (Boolean)

    When true, the tracer is kept alive during the repl session. Default is false.

  • tools (Array<LLM::Tool>)

    Extra tools to attach for the repl session

  • skills (Array<String>)

    Extra skills to attach for the repl session



48
49
50
# File 'lib/llm/sequel/agent.rb', line 48

def repl(**params)
  ctx.repl(**params)
end