Module: LLM::ActiveRecord::ActsAsAgent::InstanceMethods
- Defined in:
- lib/llm/active_record/acts_as_agent.rb
Instance Method Summary collapse
-
#llm ⇒ LLM::Provider
Returns the resolved provider instance for this record.
- #repl(**params) ⇒ void
Instance Method Details
#llm ⇒ LLM::Provider
Returns the resolved provider instance for this record.
70 71 72 73 74 75 76 |
# File 'lib/llm/active_record/acts_as_agent.rb', line 70 def llm = self.class. return @llm if @llm @llm = Utils.resolve_provider(self, , EMPTY_HASH) @llm.tracer = Utils.resolve_option(self, [:tracer]) if [:tracer] @llm end |
#repl(**params) ⇒ void
Note:
This method does not persist to the database, but it can inspect and alter runtime state in a way that is temporary.
This method returns an undefined value.
85 86 87 |
# File 'lib/llm/active_record/acts_as_agent.rb', line 85 def repl(**params) ctx.repl(**params) end |