Class: LLM::Compactor::Null
- Inherits:
-
LLM::Compactor
- Object
- LLM::Compactor
- LLM::Compactor::Null
- Defined in:
- lib/llm/compactor/null.rb
Overview
An LLM::Compactor::Null is a compactor that does nothing. It is used as the default when no compactor strategy is configured.
All methods return nil and produce no side effects.
Instance Attribute Summary
Attributes inherited from LLM::Compactor
Instance Method Summary collapse
Methods inherited from LLM::Compactor
Constructor Details
This class inherits a constructor from LLM::Compactor
Instance Method Details
#call(**opts) ⇒ nil
15 16 17 |
# File 'lib/llm/compactor/null.rb', line 15 def call(**opts) nil end |