Class: LLM::Function
- Inherits:
-
Object
- Object
- LLM::Function
- Extended by:
- Registry
- Includes:
- Tracing
- Defined in:
- lib/llm/function.rb,
lib/llm/function/fork.rb,
lib/llm/function/task.rb,
lib/llm/function/array.rb,
lib/llm/function/ractor.rb,
lib/llm/function/tracing.rb,
lib/llm/function/fork/job.rb,
lib/llm/function/registry.rb,
lib/llm/function/fork/task.rb,
lib/llm/function/fork_group.rb,
lib/llm/function/ractor/job.rb,
lib/llm/function/task_group.rb,
lib/llm/function/fiber_group.rb,
lib/llm/function/ractor/task.rb,
lib/llm/function/ractor_group.rb,
lib/llm/function/thread_group.rb,
lib/llm/function/ractor/mailbox.rb
Overview
The LLM::Function class represents a local function that can be called by an LLM.
Defined Under Namespace
Modules: Array, Fork, Ractor, Registry, Tracing Classes: FiberGroup, Return, Task, TaskGroup, ThreadGroup
Instance Attribute Summary collapse
-
#arguments ⇒ Array?
Returns function arguments.
-
#id ⇒ String?
Returns the function ID.
-
#model ⇒ String?
Returns a model name, or nil.
-
#tracer ⇒ LLM::Tracer?
Returns a tracer, or nil.
Instance Method Summary collapse
- #adapt(provider) ⇒ Hash
-
#call ⇒ LLM::Function::Return
Call the function.
-
#called? ⇒ Boolean
Returns true when a function has been called.
-
#cancel(reason: "function call cancelled") ⇒ LLM::Function::Return
Returns a value that communicates that the function call was cancelled.
-
#cancelled? ⇒ Boolean
Returns true when a function has been cancelled.
-
#define(klass = nil, &b) ⇒ void
(also: #register)
Set the function implementation.
-
#description(desc = nil) ⇒ void
Set (or get) the function description.
-
#initialize(name) {|self| ... } ⇒ Function
constructor
A new instance of Function.
-
#interrupt! ⇒ nil
(also: #cancel!)
Notifies the function runner that the call was interrupted.
-
#name(name = nil) ⇒ void
Set (or get) the function name.
-
#params {|schema| ... } ⇒ LLM::Schema::Leaf?
Set (or get) the function parameters.
-
#pending? ⇒ Boolean
Returns true when a function has neither been called nor cancelled.
-
#runner ⇒ Object
Returns the bound function runner instance.
-
#spawn(strategy) ⇒ LLM::Function::Task
Calls the function concurrently.
Methods included from Registry
clear_registry!, extended, find_by_name, lock, registry, registry_key, tool_name, unregister
Constructor Details
Instance Attribute Details
#arguments ⇒ Array?
Returns function arguments
111 112 113 |
# File 'lib/llm/function.rb', line 111 def arguments @arguments end |
#id ⇒ String?
Returns the function ID
106 107 108 |
# File 'lib/llm/function.rb', line 106 def id @id end |
#model ⇒ String?
Returns a model name, or nil
121 122 123 |
# File 'lib/llm/function.rb', line 121 def model @model end |
#tracer ⇒ LLM::Tracer?
Returns a tracer, or nil
116 117 118 |
# File 'lib/llm/function.rb', line 116 def tracer @tracer end |
Instance Method Details
#adapt(provider) ⇒ Hash
299 300 301 302 303 304 305 306 307 308 309 310 311 312 |
# File 'lib/llm/function.rb', line 299 def adapt(provider) case provider.class.to_s when "LLM::Google" {name: @name, description: @description, parameters: @params}.compact when "LLM::Anthropic" { name: @name, description: @description, input_schema: @params || {type: "object", properties: {}} }.compact else format_openai(provider) end end |
#call ⇒ LLM::Function::Return
Call the function
188 189 190 191 192 |
# File 'lib/llm/function.rb', line 188 def call call_function ensure @called = true end |
#called? ⇒ Boolean
Returns true when a function has been called
279 280 281 |
# File 'lib/llm/function.rb', line 279 def called? @called end |
#cancel(reason: "function call cancelled") ⇒ LLM::Function::Return
Returns a value that communicates that the function call was cancelled
258 259 260 261 262 |
# File 'lib/llm/function.rb', line 258 def cancel(reason: "function call cancelled") Return.new(id, name, {cancelled: true, reason:}) ensure @cancelled = true end |
#cancelled? ⇒ Boolean
Returns true when a function has been cancelled
286 287 288 |
# File 'lib/llm/function.rb', line 286 def cancelled? @cancelled end |
#define(klass = nil, &b) ⇒ void Also known as: register
This method returns an undefined value.
Set the function implementation
180 181 182 |
# File 'lib/llm/function.rb', line 180 def define(klass = nil, &b) @runner = klass || b end |
#description(desc = nil) ⇒ void
This method returns an undefined value.
Set (or get) the function description
150 151 152 153 154 155 156 |
# File 'lib/llm/function.rb', line 150 def description(desc = nil) if desc @description = desc else @description end end |
#interrupt! ⇒ nil Also known as: cancel!
Notifies the function runner that the call was interrupted. This is cooperative and only applies to runners that implement ‘on_interrupt`.
269 270 271 272 273 |
# File 'lib/llm/function.rb', line 269 def interrupt! hook = %i[on_cancel on_interrupt].find { @runner.respond_to?(_1) } @runner.public_send(hook) if hook nil end |
#name(name = nil) ⇒ void
This method returns an undefined value.
Set (or get) the function name
138 139 140 141 142 143 144 |
# File 'lib/llm/function.rb', line 138 def name(name = nil) if name @name = name.to_s else @name end end |
#params {|schema| ... } ⇒ LLM::Schema::Leaf?
Set (or get) the function parameters
162 163 164 165 166 167 168 169 170 171 172 173 174 |
# File 'lib/llm/function.rb', line 162 def params if block_given? params = yield(@schema) params = LLM::Schema.parse(params) if Hash === params if @params @params.merge!(params) else @params = params end else @params end end |
#pending? ⇒ Boolean
Returns true when a function has neither been called nor cancelled
293 294 295 |
# File 'lib/llm/function.rb', line 293 def pending? !@called && !@cancelled end |
#runner ⇒ Object
Returns the bound function runner instance.
317 318 319 320 321 |
# File 'lib/llm/function.rb', line 317 def runner runner = Class === @runner ? @runner.new : @runner runner.tracer = @tracer if runner.respond_to?(:tracer=) runner end |
#spawn(strategy) ⇒ LLM::Function::Task
Calls the function concurrently.
This is the low-level method that powers concurrent tool execution. Prefer the collection methods on Context#functions for most use cases: LLM::Function::Array#call, LLM::Function::Array#wait, or LLM::Function::Array#spawn.
221 222 223 224 225 226 227 228 229 230 231 232 233 234 235 236 237 238 239 240 241 242 243 244 245 246 247 248 |
# File 'lib/llm/function.rb', line 221 def spawn(strategy) task = case strategy when :task LLM.require "async" unless defined?(::Async) Async { call! } when :thread Thread.new { call! } when :fiber raise ArgumentError, "Fiber concurrency requires Fiber.scheduler" unless Fiber.scheduler Fiber.schedule { call! } when :fork LLM.require "xchan" unless defined?(::Chan::UNIXSocket) span = @tracer&.on_tool_start(id:, name:, arguments:, model:) Fork::Task.new(self, tracer: @tracer, span:).spawn when :ractor raise LLM::RactorError, "Ractor concurrency only supports class-based tools" unless Class === @runner if @runner.respond_to?(:skill?) && @runner.skill? raise LLM::RactorError, "Ractor concurrency does not support skill-backed tools" end span = @tracer&.on_tool_start(id:, name:, arguments:, model:) Ractor::Task.new(@runner, id, name, arguments, tracer: @tracer, span:).spawn else raise ArgumentError, "Unknown strategy: #{strategy.inspect}. Expected :thread, :task, :fiber, :fork, or :ractor" end Task.new(task, self) ensure @called = true end |