Class: LLM::Function
- Inherits:
-
Object
- Object
- LLM::Function
- Extended by:
- Registry
- Includes:
- Tracing
- Defined in:
- lib/llm/function.rb,
lib/llm/function/task.rb,
lib/llm/function/array.rb,
lib/llm/function/ractor.rb,
lib/llm/function/tracing.rb,
lib/llm/function/registry.rb,
lib/llm/function/ractor/job.rb,
lib/llm/function/task_group.rb,
lib/llm/function/fiber_group.rb,
lib/llm/function/ractor/task.rb,
lib/llm/function/ractor_group.rb,
lib/llm/function/thread_group.rb,
lib/llm/function/ractor/mailbox.rb
Overview
The LLM::Function class represents a local function that can be called by an LLM.
Defined Under Namespace
Modules: Array, Ractor, Registry, Tracing Classes: FiberGroup, Return, Task, TaskGroup, ThreadGroup
Instance Attribute Summary collapse
-
#arguments ⇒ Array?
Returns function arguments.
-
#id ⇒ String?
Returns the function ID.
-
#model ⇒ String?
Returns a model name, or nil.
-
#tracer ⇒ LLM::Tracer?
Returns a tracer, or nil.
Instance Method Summary collapse
- #adapt(provider) ⇒ Hash
-
#call ⇒ LLM::Function::Return
Call the function.
-
#called? ⇒ Boolean
Returns true when a function has been called.
-
#cancel(reason: "function call cancelled") ⇒ LLM::Function::Return
Returns a value that communicates that the function call was cancelled.
-
#cancelled? ⇒ Boolean
Returns true when a function has been cancelled.
-
#define(klass = nil, &b) ⇒ void
(also: #register)
Set the function implementation.
-
#description(desc = nil) ⇒ void
Set (or get) the function description.
-
#initialize(name) {|self| ... } ⇒ Function
constructor
A new instance of Function.
-
#interrupt! ⇒ nil
(also: #cancel!)
Notifies the function runner that the call was interrupted.
-
#name(name = nil) ⇒ void
Set (or get) the function name.
-
#params {|schema| ... } ⇒ LLM::Schema::Leaf?
Set (or get) the function parameters.
-
#pending? ⇒ Boolean
Returns true when a function has neither been called nor cancelled.
-
#spawn(strategy) ⇒ LLM::Function::Task
Calls the function concurrently.
Methods included from Registry
clear_registry!, extended, find_by_name, lock, registry, registry_key, tool_name, unregister
Constructor Details
Instance Attribute Details
#arguments ⇒ Array?
Returns function arguments
82 83 84 |
# File 'lib/llm/function.rb', line 82 def arguments @arguments end |
#id ⇒ String?
Returns the function ID
77 78 79 |
# File 'lib/llm/function.rb', line 77 def id @id end |
#model ⇒ String?
Returns a model name, or nil
92 93 94 |
# File 'lib/llm/function.rb', line 92 def model @model end |
#tracer ⇒ LLM::Tracer?
Returns a tracer, or nil
87 88 89 |
# File 'lib/llm/function.rb', line 87 def tracer @tracer end |
Instance Method Details
#adapt(provider) ⇒ Hash
263 264 265 266 267 268 269 270 271 272 273 274 275 276 |
# File 'lib/llm/function.rb', line 263 def adapt(provider) case provider.class.to_s when "LLM::Google" {name: @name, description: @description, parameters: @params}.compact when "LLM::Anthropic" { name: @name, description: @description, input_schema: @params || {type: "object", properties: {}} }.compact else format_openai(provider) end end |
#call ⇒ LLM::Function::Return
Call the function
159 160 161 162 163 |
# File 'lib/llm/function.rb', line 159 def call call_function ensure @called = true end |
#called? ⇒ Boolean
Returns true when a function has been called
243 244 245 |
# File 'lib/llm/function.rb', line 243 def called? @called end |
#cancel(reason: "function call cancelled") ⇒ LLM::Function::Return
Returns a value that communicates that the function call was cancelled
222 223 224 225 226 |
# File 'lib/llm/function.rb', line 222 def cancel(reason: "function call cancelled") Return.new(id, name, {cancelled: true, reason:}) ensure @cancelled = true end |
#cancelled? ⇒ Boolean
Returns true when a function has been cancelled
250 251 252 |
# File 'lib/llm/function.rb', line 250 def cancelled? @cancelled end |
#define(klass = nil, &b) ⇒ void Also known as: register
This method returns an undefined value.
Set the function implementation
151 152 153 |
# File 'lib/llm/function.rb', line 151 def define(klass = nil, &b) @runner = klass || b end |
#description(desc = nil) ⇒ void
This method returns an undefined value.
Set (or get) the function description
121 122 123 124 125 126 127 |
# File 'lib/llm/function.rb', line 121 def description(desc = nil) if desc @description = desc else @description end end |
#interrupt! ⇒ nil Also known as: cancel!
Notifies the function runner that the call was interrupted. This is cooperative and only applies to runners that implement ‘on_interrupt`.
233 234 235 236 237 |
# File 'lib/llm/function.rb', line 233 def interrupt! hook = %i[on_cancel on_interrupt].find { @runner.respond_to?(_1) } @runner.public_send(hook) if hook nil end |
#name(name = nil) ⇒ void
This method returns an undefined value.
Set (or get) the function name
109 110 111 112 113 114 115 |
# File 'lib/llm/function.rb', line 109 def name(name = nil) if name @name = name.to_s else @name end end |
#params {|schema| ... } ⇒ LLM::Schema::Leaf?
Set (or get) the function parameters
133 134 135 136 137 138 139 140 141 142 143 144 145 |
# File 'lib/llm/function.rb', line 133 def params if block_given? params = yield(@schema) params = LLM::Schema.parse(params) if Hash === params if @params @params.merge!(params) else @params = params end else @params end end |
#pending? ⇒ Boolean
Returns true when a function has neither been called nor cancelled
257 258 259 |
# File 'lib/llm/function.rb', line 257 def pending? !@called && !@cancelled end |
#spawn(strategy) ⇒ LLM::Function::Task
Calls the function concurrently.
This is the low-level method that powers concurrent tool execution. Prefer the collection methods on Context#functions for most use cases: LLM::Function::Array#call, LLM::Function::Array#wait, or LLM::Function::Array#spawn.
190 191 192 193 194 195 196 197 198 199 200 201 202 203 204 205 206 207 208 209 210 211 212 |
# File 'lib/llm/function.rb', line 190 def spawn(strategy) task = case strategy when :task require "async" unless defined?(::Async) Async { call } when :thread Thread.new { call } when :fiber Fiber.new do call ensure Fiber.yield end.tap(&:resume) when :ractor raise ArgumentError, "Ractor concurrency only supports class-based tools" unless Class === @runner Ractor::Task.new(@runner, id, name, arguments) else raise ArgumentError, "Unknown strategy: #{strategy.inspect}. Expected :thread, :task, :fiber, or :ractor" end Task.new(task, self) ensure @called = true end |