Class: LLM::Function::Task

Inherits:
Object
  • Object
show all
Defined in:
lib/llm/function/task.rb

Overview

The Task class wraps a single concurrent function call and provides a small, uniform interface across threads, fibers, and async tasks.

Instance Attribute Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(task, function = nil) ⇒ LLM::Function::Task

Parameters:



20
21
22
23
# File 'lib/llm/function/task.rb', line 20

def initialize(task, function = nil)
  @task = task
  @function = function
end

Instance Attribute Details

#functionLLM::Function? (readonly)

Returns:



14
15
16
# File 'lib/llm/function/task.rb', line 14

def function
  @function
end

#taskObject (readonly)

Returns:



10
11
12
# File 'lib/llm/function/task.rb', line 10

def task
  @task
end

Instance Method Details

#alive?Boolean

Returns:

  • (Boolean)


27
28
29
30
# File 'lib/llm/function/task.rb', line 27

def alive?
  return task.alive? if task.respond_to?(:alive?)
  false
end

#waitLLM::Function::Return Also known as: value



34
35
36
37
38
39
40
41
42
43
# File 'lib/llm/function/task.rb', line 34

def wait
  if Thread === task
    task.value
  elsif Fiber === task
    task.resume if task.alive?
    task.value
  else
    task.wait
  end
end