Class: LLM::Provider Abstract
- Inherits:
-
Object
- Object
- LLM::Provider
- Includes:
- Transport::Execution
- Defined in:
- lib/llm/provider.rb
Overview
Instance Method Summary collapse
-
#adapt_function(fn) ⇒ Hash
abstract
Adapt a Function to the provider-specific tool schema.
-
#assistant_role ⇒ String
Returns the role of the assistant in the conversation.
-
#audio ⇒ LLM::OpenAI::Audio
Returns an interface to the audio API.
-
#chat(prompt, params = {}) ⇒ LLM::Context
Starts a new chat powered by the chat completions API.
-
#complete(prompt, params = {}) ⇒ LLM::Response
Provides an interface to the chat completions API.
-
#default_model ⇒ String
Returns the default model for chat completions.
- #developer_role ⇒ Symbol
-
#embed(input, model: nil, **params) ⇒ LLM::Response
Provides an embedding.
-
#files ⇒ LLM::OpenAI::Files
Returns an interface to the files API.
-
#images ⇒ LLM::OpenAI::Images, LLM::Google::Images
Returns an interface to the images API.
-
#initialize(key:, host:, port: 443, timeout: 900, ssl: true, base_path: "", persistent: false, transport: nil) ⇒ Provider
constructor
A new instance of Provider.
-
#inspect ⇒ String
Returns an inspection of the provider object.
-
#interrupt!(owner) ⇒ nil
(also: #cancel!)
Interrupt the active request, if any.
-
#key? ⇒ Boolean
Returns true when an API key is configured.
-
#models ⇒ LLM::OpenAI::Models
Returns an interface to the models API.
-
#moderations ⇒ LLM::OpenAI::Moderations
Returns an interface to the moderations API.
-
#name ⇒ Symbol
Returns the provider's name.
- #ocr ⇒ LLM::Response
-
#request_owner ⇒ Object
private
Returns the current request owner used by the transport.
-
#respond(prompt, params = {}) ⇒ LLM::Context
Starts a new chat powered by the responses API.
-
#responses ⇒ LLM::OpenAI::Responses
Compared to the chat completions API, the responses API can require less bandwidth on each turn, maintain state server-side, and produce faster responses.
-
#schema ⇒ LLM::Schema
Returns an object that can generate a JSON schema.
-
#server_tool(name, options = {}) ⇒ LLM::ServerTool
Returns a tool provided by a provider.
-
#server_tools ⇒ String => LLM::ServerTool
Returns all known tools provided by a provider.
- #system_role ⇒ Symbol
- #tool_role ⇒ Symbol
-
#tracer ⇒ LLM::Tracer
Returns the current scoped tracer override or provider default tracer.
-
#tracer=(tracer) ⇒ void
Set the provider's default tracer This tracer is shared by the provider instance and becomes the fallback whenever no scoped override is active.
- #user_role ⇒ Symbol
-
#vector_stores ⇒ LLM::OpenAI::VectorStore
Returns an interface to the vector stores API.
-
#web_search(query:) ⇒ LLM::Response
Provides a web search capability.
-
#with(headers:) ⇒ LLM::Provider
Add one or more headers to all requests.
-
#with_tracer(tracer) { ... } ⇒ Object
Override the tracer for the current fiber while the block runs.
Constructor Details
#initialize(key:, host:, port: 443, timeout: 900, ssl: true, base_path: "", persistent: false, transport: nil) ⇒ Provider
Returns a new instance of Provider.
30 31 32 33 34 35 36 37 38 39 40 41 |
# File 'lib/llm/provider.rb', line 30 def initialize(key:, host:, port: 443, timeout: 900, ssl: true, base_path: "", persistent: false, transport: nil) @key = key @host = host @port = port @timeout = timeout @ssl = ssl @base_path = LLM::Utils.normalize_base_path(base_path) @base_uri = URI("#{ssl ? "https" : "http"}://#{host}:#{port}/") @headers = {"User-Agent" => "llm.rb v#{LLM::VERSION}"} @transport = LLM::Transport::Utils.resolve_transport(host:, port:, timeout:, ssl:, transport:, persistent:) @monitor = Monitor.new end |
Instance Method Details
#adapt_function(fn) ⇒ Hash
Adapt a Function to the provider-specific tool schema.
361 362 363 |
# File 'lib/llm/provider.rb', line 361 def adapt_function(fn) raise NotImplementedError end |
#assistant_role ⇒ String
Returns the role of the assistant in the conversation. Usually "assistant" or "model"
189 190 191 |
# File 'lib/llm/provider.rb', line 189 def assistant_role raise NotImplementedError end |
#audio ⇒ LLM::OpenAI::Audio
Returns an interface to the audio API
153 154 155 |
# File 'lib/llm/provider.rb', line 153 def audio raise NotImplementedError end |
#chat(prompt, params = {}) ⇒ LLM::Context
Starts a new chat powered by the chat completions API
116 117 118 119 |
# File 'lib/llm/provider.rb', line 116 def chat(prompt, params = {}) role = params.delete(:role) LLM::Context.new(self, params).talk(prompt, role:) end |
#complete(prompt, params = {}) ⇒ LLM::Response
Provides an interface to the chat completions API. Most users should use Context#talk or Agent#talk instead.
107 108 109 |
# File 'lib/llm/provider.rb', line 107 def complete(prompt, params = {}) raise NotImplementedError end |
#default_model ⇒ String
Returns the default model for chat completions
196 197 198 |
# File 'lib/llm/provider.rb', line 196 def default_model raise NotImplementedError end |
#developer_role ⇒ Symbol
274 275 276 |
# File 'lib/llm/provider.rb', line 274 def developer_role :developer end |
#embed(input, model: nil, **params) ⇒ LLM::Response
Provides an embedding
71 72 73 |
# File 'lib/llm/provider.rb', line 71 def (input, model: nil, **params) raise NotImplementedError end |
#files ⇒ LLM::OpenAI::Files
Returns an interface to the files API
160 161 162 |
# File 'lib/llm/provider.rb', line 160 def files raise NotImplementedError end |
#images ⇒ LLM::OpenAI::Images, LLM::Google::Images
Returns an interface to the images API
146 147 148 |
# File 'lib/llm/provider.rb', line 146 def images raise NotImplementedError end |
#inspect ⇒ String
The secret key is redacted in inspect for security reasons
Returns an inspection of the provider object
47 48 49 |
# File 'lib/llm/provider.rb', line 47 def inspect "#<#{LLM::Utils.object_id(self)} @key=[REDACTED] @transport=#{transport.inspect} @tracer=#{tracer.inspect}>" end |
#interrupt!(owner) ⇒ nil Also known as: cancel!
Interrupt the active request, if any.
335 336 337 |
# File 'lib/llm/provider.rb', line 335 def interrupt!(owner) transport.interrupt!(owner) end |
#key? ⇒ Boolean
Returns true when an API key is configured
351 352 353 |
# File 'lib/llm/provider.rb', line 351 def key? @key != nil && @key.to_s.strip.size > 0 end |
#models ⇒ LLM::OpenAI::Models
Returns an interface to the models API
167 168 169 |
# File 'lib/llm/provider.rb', line 167 def models raise NotImplementedError end |
#moderations ⇒ LLM::OpenAI::Moderations
Returns an interface to the moderations API
174 175 176 |
# File 'lib/llm/provider.rb', line 174 def moderations raise NotImplementedError end |
#name ⇒ Symbol
Returns the provider's name
56 57 58 |
# File 'lib/llm/provider.rb', line 56 def name raise NotImplementedError end |
#ocr ⇒ LLM::Response
This feature is not implemented by all providers, and it will raise NotImplementedError for providers that do not support it.
81 82 83 |
# File 'lib/llm/provider.rb', line 81 def ocr(...) raise NotImplementedError end |
#request_owner ⇒ Object
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
Returns the current request owner used by the transport.
344 345 346 |
# File 'lib/llm/provider.rb', line 344 def request_owner transport.request_owner end |
#respond(prompt, params = {}) ⇒ LLM::Context
Starts a new chat powered by the responses API
127 128 129 130 |
# File 'lib/llm/provider.rb', line 127 def respond(prompt, params = {}) role = params.delete(:role) LLM::Context.new(self, params).respond(prompt, role:) end |
#responses ⇒ LLM::OpenAI::Responses
Compared to the chat completions API, the responses API can require less bandwidth on each turn, maintain state server-side, and produce faster responses.
139 140 141 |
# File 'lib/llm/provider.rb', line 139 def responses raise NotImplementedError end |
#schema ⇒ LLM::Schema
Returns an object that can generate a JSON schema
203 204 205 |
# File 'lib/llm/provider.rb', line 203 def schema LLM::Schema.new end |
#server_tool(name, options = {}) ⇒ LLM::ServerTool
OpenAI, Anthropic, and Gemini provide platform-tools for things like web search, and more.
Returns a tool provided by a provider.
246 247 248 |
# File 'lib/llm/provider.rb', line 246 def server_tool(name, = {}) LLM::ServerTool.new(name, , self) end |
#server_tools ⇒ String => LLM::ServerTool
This method might be outdated, and the LLM::Provider#server_tool method can be used if a tool is not found here.
Returns all known tools provided by a provider.
229 230 231 |
# File 'lib/llm/provider.rb', line 229 def server_tools {} end |
#system_role ⇒ Symbol
268 269 270 |
# File 'lib/llm/provider.rb', line 268 def system_role :system end |
#tool_role ⇒ Symbol
280 281 282 |
# File 'lib/llm/provider.rb', line 280 def tool_role :tool end |
#tracer ⇒ LLM::Tracer
Returns the current scoped tracer override or provider default tracer
287 288 289 |
# File 'lib/llm/provider.rb', line 287 def tracer weakmap[self] || @tracer || LLM::Tracer::Null.new(self) end |
#tracer=(tracer) ⇒ void
This method returns an undefined value.
Set the provider's default tracer This tracer is shared by the provider instance and becomes the fallback whenever no scoped override is active.
301 302 303 |
# File 'lib/llm/provider.rb', line 301 def tracer=(tracer) @tracer = tracer || LLM::Tracer::Null.new(self) end |
#user_role ⇒ Symbol
262 263 264 |
# File 'lib/llm/provider.rb', line 262 def user_role :user end |
#vector_stores ⇒ LLM::OpenAI::VectorStore
Returns an interface to the vector stores API
181 182 183 |
# File 'lib/llm/provider.rb', line 181 def vector_stores raise NotImplementedError end |
#web_search(query:) ⇒ LLM::Response
Provides a web search capability
256 257 258 |
# File 'lib/llm/provider.rb', line 256 def web_search(query:) raise NotImplementedError end |
#with(headers:) ⇒ LLM::Provider
Add one or more headers to all requests
217 218 219 220 221 |
# File 'lib/llm/provider.rb', line 217 def with(headers:) lock do tap { @headers.merge!(headers) } end end |
#with_tracer(tracer) { ... } ⇒ Object
Override the tracer for the current fiber while the block runs. This is useful when you want per-request or per-turn tracing without replacing the provider's default tracer.
316 317 318 319 320 321 322 323 324 325 326 327 328 329 |
# File 'lib/llm/provider.rb', line 316 def with_tracer(tracer) had_override = weakmap.key?(self) previous = weakmap[self] weakmap[self] = tracer || LLM::Tracer::Null.new(self) yield ensure if had_override weakmap[self] = previous elsif weakmap.respond_to?(:delete) weakmap.delete(self) else weakmap[self] = nil end end |