Class: LLM::Repl

Inherits:
Object
  • Object
show all
Defined in:
lib/llm/repl.rb,
lib/llm/repl/bar.rb,
lib/llm/repl/input.rb,
lib/llm/repl/status.rb,
lib/llm/repl/stream.rb,
lib/llm/repl/walker.rb,
lib/llm/repl/window.rb,
lib/llm/repl/command.rb,
lib/llm/repl/markdown.rb,
lib/llm/repl/transcript.rb,
lib/llm/repl/commands/exit.rb,
lib/llm/repl/commands/help.rb,
lib/llm/repl/commands/compact.rb

Overview

The LLM::Repl class provides a small read-eval-print loop around an instance of LLM::Agent.

It can be used to keep talking to an agent after it has been set up or has performed a task. This can be useful when you want to confirm the agent handled the task correctly, or for it to correct course after a mistake was made.

Defined Under Namespace

Classes: Bar, Command, Help, Input, Markdown, Status, Stream, Transcript, Walker, Window

Instance Attribute Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(agent:, name: nil, tools: [], skills: [], path: nil) ⇒ LLM::Repl

Parameters:

  • agent (LLM::Agent)
  • name (String, nil) (defaults to: nil)

    The agent's name (optional)

  • path (String, nil) (defaults to: nil)

    The path where to maintain runtime state

  • tools (Array<LLM::Tool>) (defaults to: [])

    Zero or more tools

  • skills (Array<String>) (defaults to: [])

    Zero or more skills



44
45
46
47
48
49
50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
# File 'lib/llm/repl.rb', line 44

def initialize(agent:, name: nil, tools: [], skills: [], path: nil)
  @path = path
  @name = name || "agent"
  @agent = configure(agent:, path:)
  @provider = agent.llm.name
  @status = Status.new(@agent)
  @transcript = Transcript.new
  @input = Input.new(self, height: 3)
  @window = Window.new(@status, @transcript, @input)
  @thread = nil
  @queue = Queue.new
  @stream = Stream.new(self, @queue)
  @skills = skills.map do |path|
    ##
    # I'm not sure it would make sense to expose
    # the underlying context or not. In the meantime,
    # this works and meets the expectations of the
    # LLM::Skill class.
    ctx = agent.instance_variable_get(:@ctx)
    LLM::Skill.load(path).to_tool(ctx)
  end
  @tools = [agent.params[:tools], @skills, tools].flatten.compact
end

Instance Attribute Details

#agentObject (readonly)

Returns the value of attribute agent.



28
29
30
# File 'lib/llm/repl.rb', line 28

def agent
  @agent
end

#inputObject (readonly)

Returns the value of attribute input.



28
29
30
# File 'lib/llm/repl.rb', line 28

def input
  @input
end

#nameObject (readonly)

Returns the value of attribute name.



28
29
30
# File 'lib/llm/repl.rb', line 28

def name
  @name
end

#pathObject (readonly)

Returns the value of attribute path.



28
29
30
# File 'lib/llm/repl.rb', line 28

def path
  @path
end

#providerObject (readonly)

Returns the value of attribute provider.



28
29
30
# File 'lib/llm/repl.rb', line 28

def provider
  @provider
end

#statusObject

Returns the value of attribute status.



28
29
30
# File 'lib/llm/repl.rb', line 28

def status
  @status
end

#streamObject (readonly)

Returns the value of attribute stream.



28
29
30
# File 'lib/llm/repl.rb', line 28

def stream
  @stream
end

#threadObject (readonly)

Returns the value of attribute thread.



28
29
30
# File 'lib/llm/repl.rb', line 28

def thread
  @thread
end

#toolsObject (readonly)

Returns the value of attribute tools.



28
29
30
# File 'lib/llm/repl.rb', line 28

def tools
  @tools
end

#transcriptObject (readonly)

Returns the value of attribute transcript.



28
29
30
# File 'lib/llm/repl.rb', line 28

def transcript
  @transcript
end

#windowObject (readonly)

Returns the value of attribute window.



28
29
30
# File 'lib/llm/repl.rb', line 28

def window
  @window
end

Instance Method Details

#markdown(chars) ⇒ void

This method returns an undefined value.

Parameters:

  • chars (String)


100
101
102
103
# File 'lib/llm/repl.rb', line 100

def markdown(chars)
  transcript.markdown(chars)
  window.redraw
end

#startvoid

This method returns an undefined value.



70
71
72
73
74
75
76
77
78
79
80
81
82
83
84
85
86
# File 'lib/llm/repl.rb', line 70

def start
  window.open do
    catch(:exit) do
      loop do
        now = Process.clock_gettime(Process::CLOCK_MONOTONIC)
        case input.on_char(window, input.paste? ? window.read_paste : window.getch, now)
        when :submit then submit
        when Symbol then window.redraw
        else
          window.redraw
          read!
          sleep 0.01
        end
      end
    end
  end
end

#thinking_textString

Returns:

  • (String)


115
116
117
# File 'lib/llm/repl.rb', line 115

def thinking_text
  "thinking • Esc to cancel"
end

#write(chars, attrs = nil) ⇒ void

This method returns an undefined value.

Parameters:

  • chars (String)
  • attrs (Object) (defaults to: nil)


92
93
94
95
# File 'lib/llm/repl.rb', line 92

def write(chars, attrs = nil)
  transcript.write(chars, attrs)
  window.redraw
end