Class: LLM::Schema

Inherits:
Object
  • Object
show all
Extended by:
Parser
Defined in:
lib/llm/schema.rb,
lib/llm/schema/enum.rb,
lib/llm/schema/leaf.rb,
lib/llm/schema/null.rb,
lib/llm/schema/array.rb,
lib/llm/schema/all_of.rb,
lib/llm/schema/any_of.rb,
lib/llm/schema/number.rb,
lib/llm/schema/object.rb,
lib/llm/schema/one_of.rb,
lib/llm/schema/parser.rb,
lib/llm/schema/string.rb,
lib/llm/schema/boolean.rb,
lib/llm/schema/integer.rb,
lib/llm/schema/version.rb

Overview

The LLM::Schema class represents a JSON schema, and provides methods that let you describe and produce a schema that can be used in various contexts that include the validation and generation of JSON data.

Examples:

JavaScript-style

schema = LLM::Schema.new
schema.object({
  name: schema.string.enum("John", "Jane").required,
  age: schema.integer.required,
  hobbies: schema.array(schema.string).required,
  address: schema.object({street: schema.string}).required,
})

Ruby-style

class Address < LLM::Schema
  property :street, String, "Street address", required: true
end

class Person < LLM::Schema
  property :name, String, "Person's name", required: true
  property :age, Integer, "Person's age", required: true
  property :hobbies, Array[String], "Person's hobbies", required: true
  property :address, Address, "Person's address", required: true
end

See Also:

Defined Under Namespace

Modules: Parser, Utils Classes: AllOf, AnyOf, Array, Boolean, Enum, Integer, Leaf, Null, Number, Object, OneOf, String

Constant Summary collapse

VERSION =
"0.1.0"

Constants included from Parser

Parser::METADATA_KEYS

Class Method Summary collapse

Instance Method Summary collapse

Methods included from Parser

parse

Class Method Details

.inherited(klass) ⇒ void

This method returns an undefined value.

Configures a monitor for a subclass



82
83
84
85
86
# File 'lib/llm/schema.rb', line 82

def self.inherited(klass)
  LLM.lock(:inherited) do
    klass.instance_eval { @__monitor = Monitor.new }
  end
end

.objectLLM::Schema::Object

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

Returns:



118
119
120
121
122
# File 'lib/llm/schema.rb', line 118

def self.object
  lock do
    @object ||= schema.object({})
  end
end

.property(name, type, description, options = {}) ⇒ Object

Parameters:

  • name (String)

    The property name

  • type (Class)

    The property type

  • description (String)

    The property description

  • options (Hash) (defaults to: {})

    A hash of options



97
98
99
100
101
102
103
104
# File 'lib/llm/schema.rb', line 97

def self.property(name, type, description, options = {})
  lock do
    prop = Utils.resolve(schema, type)
    options = {description:}.merge(options)
    options.each { (_2 == true) ? prop.public_send(_1) : prop.public_send(_1, *_2) }
    object[name] = prop
  end
end

.schemaLLM::Schema

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

Returns:



109
110
111
112
113
# File 'lib/llm/schema.rb', line 109

def self.schema
  lock do
    @schema ||= LLM::Schema.new
  end
end

Instance Method Details

#all_of(*values) ⇒ LLM::Schema::AllOf

Returns an allOf union

Parameters:

Returns:



159
160
161
# File 'lib/llm/schema.rb', line 159

def all_of(*values)
  AllOf.new(values)
end

#any_of(*values) ⇒ LLM::Schema::AnyOf

Returns an anyOf union

Parameters:

Returns:



151
152
153
# File 'lib/llm/schema.rb', line 151

def any_of(*values)
  AnyOf.new(values)
end

#array(*items) ⇒ LLM::Schema::Array

Returns an array

Parameters:

  • items (Array)

    An array of items

Returns:



143
144
145
# File 'lib/llm/schema.rb', line 143

def array(*items)
  Array.new(*items)
end

#booleanLLM::Schema::Boolean

Returns a boolean



195
196
197
# File 'lib/llm/schema.rb', line 195

def boolean
  Boolean.new
end

#integerLLM::Schema::Integer

Returns an integer



188
189
190
# File 'lib/llm/schema.rb', line 188

def integer
  Integer.new
end

#nullLLM::Schema::Null

Returns null

Returns:



202
203
204
# File 'lib/llm/schema.rb', line 202

def null
  Null.new
end

#numberLLM::Schema::Number

Returns a number

Returns:



181
182
183
# File 'lib/llm/schema.rb', line 181

def number
  Number.new
end

#object(properties) ⇒ LLM::Schema::Object

Returns an object

Parameters:

  • properties (Hash)

    A hash of properties

Returns:



135
136
137
# File 'lib/llm/schema.rb', line 135

def object(properties)
  Object.new(properties)
end

#one_of(*values) ⇒ LLM::Schema::OneOf

Returns a oneOf union

Parameters:

Returns:



167
168
169
# File 'lib/llm/schema.rb', line 167

def one_of(*values)
  OneOf.new(values)
end

#stringLLM::Schema::String

Returns a string

Returns:



174
175
176
# File 'lib/llm/schema.rb', line 174

def string
  String.new
end