Class: LLM::Skill

Inherits:
Object
  • Object
show all
Defined in:
lib/llm/skill.rb

Overview

LLM::Skill represents a directory-backed packaged capability. A skill directory must contain a ‘SKILL.md` file with YAML frontmatter. Skills can expose themselves as normal LLM::Tool classes through #to_tool. This keeps skills on the same execution path as local tools.

Instance Attribute Summary collapse

Class Method Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(path) ⇒ LLM::Skill

Parameters:

  • path (String)

    The path to a directory



52
53
54
55
56
57
58
59
# File 'lib/llm/skill.rb', line 52

def initialize(path)
  @path = path.to_s
  @name = ::File.basename(@path)
  @description = "Skill: #{@name}"
  @instructions = ""
  @frontmatter = LLM::Object.from({})
  @tools = []
end

Instance Attribute Details

#descriptionString (readonly)

Returns the skill description.

Returns:

  • (String)


31
32
33
# File 'lib/llm/skill.rb', line 31

def description
  @description
end

#frontmatterLLM::Object (readonly)

Returns the skill frontmatter.

Returns:



41
42
43
# File 'lib/llm/skill.rb', line 41

def frontmatter
  @frontmatter
end

#instructionsString (readonly)

Returns the skill instructions.

Returns:

  • (String)


36
37
38
# File 'lib/llm/skill.rb', line 36

def instructions
  @instructions
end

#nameString (readonly)

Returns the skill name.

Returns:

  • (String)


26
27
28
# File 'lib/llm/skill.rb', line 26

def name
  @name
end

#pathString (readonly)

Returns the skill directory.

Returns:

  • (String)


21
22
23
# File 'lib/llm/skill.rb', line 21

def path
  @path
end

#toolsArray<Class<LLM::Tool>> (readonly)

Returns the skill tools.

Returns:



46
47
48
# File 'lib/llm/skill.rb', line 46

def tools
  @tools
end

Class Method Details

.load(path) ⇒ LLM::Skill

Load a skill from a directory.

Parameters:

  • path (String, Pathname)

Returns:



14
15
16
# File 'lib/llm/skill.rb', line 14

def self.load(path)
  new(path).tap(&:load!)
end

Instance Method Details

#call(ctx) ⇒ Hash

Execute the skill by wrapping it in a small agent with the skill instructions. The context is bound explicitly by the caller so the nested agent can inherit context-level behavior such as streaming.

Parameters:

Returns:

  • (Hash)


76
77
78
79
80
81
82
83
84
85
86
# File 'lib/llm/skill.rb', line 76

def call(ctx)
  instructions, tools = self.instructions, self.tools
  params = ctx.params.merge(mode: ctx.mode).reject { [:tools, :schema].include?(_1) }
  agent = Class.new(LLM::Agent) do
    instructions(instructions)
    tools(*tools)
  end.new(ctx.llm, params)
  agent.messages.concat(messages_for(ctx))
  res = agent.talk("Solve the user's query.")
  {content: res.content}
end

#load!LLM::Skill

Load and parse the skill.

Returns:



64
65
66
67
68
# File 'lib/llm/skill.rb', line 64

def load!
  path = ::File.join(@path, "SKILL.md")
  parse(::File.read(path))
  self
end

#to_tool(ctx) ⇒ Class<LLM::Tool>

Expose the skill as a normal LLM::Tool. The context is bound explicitly when the tool class is built.

Parameters:

Returns:



93
94
95
96
97
98
99
100
101
102
103
# File 'lib/llm/skill.rb', line 93

def to_tool(ctx)
  skill = self
  Class.new(LLM::Tool) do
    name skill.name
    description skill.description

    define_method(:call) do
      skill.call(ctx)
    end
  end
end